@@ -29,8 +29,6 @@
 
 #undef M_DEBUG
 #undef M_GUARD
-#undef VERSION
-#undef PACKAGE
 #undef PRINTABLE_OS_NAME
 #undef IS_DEVELOPMENT_VERSION
 
@@ -53,6 +51,7 @@
 
 #undef HAVE_STPCPY
 
+#undef HAVE_MLOCK
 
 #undef BIG_ENDIAN_HOST
 #undef LITTLE_ENDIAN_HOST
@@ -73,6 +72,9 @@
 /* Linux has an ioctl */
 #undef HAVE_DEV_RANDOM_IOCTL
 
+/* see cipher/rndegd.c */
+#undef EGD_SOCKET_NAME
+
 
 #undef USE_DYNAMIC_LINKING
 #undef HAVE_DL_DLOPEN
@@ -91,6 +93,13 @@
 /* defined if we run on some of the PCDOS like systems (DOS, Windoze. OS/2)
  * with special properties like no file modes */
 #undef HAVE_DOSISH_SYSTEM
+/* because the Unix gettext has to much overhead on MingW32 systems
+ * and these systems lack Posix functions, we use a simplified version
+ * of gettext */
+#undef USE_SIMPLE_GETTEXT
+/* At some point in the system we need to know that we use the Windows
+ * random module. */
+#undef USE_STATIC_RNDW32
 
 #undef USE_CAPABILITIES
 
