@@ -30,6 +30,9 @@
 #include "cipher.h"
 #include "elgamal.h"
 #include "dsa.h"
+#if 0
+#include "rsa.h"
+#endif
 #include "dynload.h"
 
 
@@ -54,7 +57,7 @@ struct pubkey_table_s {
 };
 
 static struct pubkey_table_s pubkey_table[TABLE_SIZE];
-
+static int disabled_algos[TABLE_SIZE];
 
 
 static int
@@ -89,6 +92,8 @@ dummy_get_nbits( int algo, MPI *pkey )
 
 /****************
  * Put the static entries into the table.
+ * This is out constructor function which fill the table
+ * of algorithms with the one we have statically linked.
  */
 static void
 setup_pubkey_table(void)
@@ -148,6 +153,60 @@ setup_pubkey_table(void)
 	BUG();
     i++;
 
+  #if 0
+    pubkey_table[i].algo = PUBKEY_ALGO_RSA;
+    pubkey_table[i].name = rsa_get_info( pubkey_table[i].algo,
+					 &pubkey_table[i].npkey,
+					 &pubkey_table[i].nskey,
+					 &pubkey_table[i].nenc,
+					 &pubkey_table[i].nsig,
+					 &pubkey_table[i].use );
+    pubkey_table[i].generate	     = rsa_generate;
+    pubkey_table[i].check_secret_key = rsa_check_secret_key;
+    pubkey_table[i].encrypt	     = rsa_encrypt;
+    pubkey_table[i].decrypt	     = rsa_decrypt;
+    pubkey_table[i].sign	     = rsa_sign;
+    pubkey_table[i].verify	     = rsa_verify;
+    pubkey_table[i].get_nbits	     = rsa_get_nbits;
+    if( !pubkey_table[i].name )
+	BUG();
+    i++;
+    pubkey_table[i].algo = PUBKEY_ALGO_RSA_E;
+    pubkey_table[i].name = rsa_get_info( pubkey_table[i].algo,
+					 &pubkey_table[i].npkey,
+					 &pubkey_table[i].nskey,
+					 &pubkey_table[i].nenc,
+					 &pubkey_table[i].nsig,
+					 &pubkey_table[i].use );
+    pubkey_table[i].generate	     = rsa_generate;
+    pubkey_table[i].check_secret_key = rsa_check_secret_key;
+    pubkey_table[i].encrypt	     = rsa_encrypt;
+    pubkey_table[i].decrypt	     = rsa_decrypt;
+    pubkey_table[i].sign	     = dummy_sign;
+    pubkey_table[i].verify	     = dummy_verify;
+    pubkey_table[i].get_nbits	     = rsa_get_nbits;
+    if( !pubkey_table[i].name )
+	BUG();
+    i++;
+    pubkey_table[i].algo = PUBKEY_ALGO_RSA_S;
+    pubkey_table[i].name = rsa_get_info( pubkey_table[i].algo,
+					 &pubkey_table[i].npkey,
+					 &pubkey_table[i].nskey,
+					 &pubkey_table[i].nenc,
+					 &pubkey_table[i].nsig,
+					 &pubkey_table[i].use );
+    pubkey_table[i].generate	     = rsa_generate;
+    pubkey_table[i].check_secret_key = rsa_check_secret_key;
+    pubkey_table[i].encrypt	     = dummy_encrypt;
+    pubkey_table[i].decrypt	     = dummy_decrypt;
+    pubkey_table[i].sign	     = rsa_sign;
+    pubkey_table[i].verify	     = rsa_verify;
+    pubkey_table[i].get_nbits	     = rsa_get_nbits;
+    if( !pubkey_table[i].name )
+	BUG();
+    i++;
+  #endif
+
     for( ; i < TABLE_SIZE; i++ )
 	pubkey_table[i].name = NULL;
 }
@@ -265,6 +324,20 @@ pubkey_algo_to_string( int algo )
 }
 
 
+void
+disable_pubkey_algo( int algo )
+{
+    int i;
+
+    for(i=0; i < DIM(disabled_algos); i++ ) {
+	if( !disabled_algos[i] || disabled_algos[i] == algo ) {
+	    disabled_algos[i] = algo;
+	    return;
+	}
+    }
+    log_fatal("can't disable pubkey algo %d: table full\n", algo );
+}
+
 
 int
 check_pubkey_algo( int algo )
@@ -289,6 +362,11 @@ check_pubkey_algo2( int algo, unsigned use )
 		if( (use & PUBKEY_USAGE_ENC)
 		    && !(pubkey_table[i].use & PUBKEY_USAGE_ENC) )
 		    return G10ERR_WR_PUBKEY_ALGO;
+
+		for(i=0; i < DIM(disabled_algos); i++ ) {
+		    if( disabled_algos[i] == algo )
+			return G10ERR_PUBKEY_ALGO;
+		}
 		return 0; /* okay */
 	    }
     } while( load_pubkey_modules() );
