@@ -33,6 +33,13 @@
 #include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
+#if 0
+  #ifdef HAVE_LINUX_RANDOM_H
+    #include <sys/ioctl.h>
+    #include <asm/types.h>
+    #include <linux/random.h>
+  #endif
+#endif
 #include "types.h"
 #include "util.h"
 #include "ttyio.h"
@@ -48,6 +55,19 @@ static int open_device( const char *name, int minor );
 static int gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level );
 
+#if 0
+#ifdef HAVE_DEV_RANDOM_IOCTL
+static ulong
+get_entropy_count( int fd )
+{
+    ulong count;
+
+    if( ioctl( fd, RNDGETENTCNT, &count ) == -1 )
+	g10_log_fatal("ioctl(RNDGETENTCNT) failed: %s\n", strerror(errno) );
+    return count;
+}
+#endif
+#endif
 
 /****************
  * Used to open the Linux and xBSD /dev/random devices
@@ -69,6 +89,10 @@ open_device( const char *name, int minor )
 }
 
 
+/****************
+ * Note:  Using a level of 0 should never block and better add nothing
+ * to the pool.  This is easy to accomplish with /dev/urandom.
+ */
 static int
 gather_random( void (*add)(const void*, size_t, int), int requester,
 					  size_t length, int level )
@@ -86,11 +110,18 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 	fd = fd_random;
     }
     else {
+	/* this will also be used for elve 0 but by using /dev/urandom
+	 * we can be sure that oit will never block. */
 	if( fd_urandom == -1 )
 	    fd_urandom = open_device( NAME_OF_DEV_URANDOM, 9 );
 	fd = fd_urandom;
     }
 
+  #if 0
+  #ifdef HAVE_DEV_RANDOM_IOCTL
+    g10_log_info("entropy count of %d is %lu\n", fd, get_entropy_count(fd) );
+  #endif
+  #endif
     while( length ) {
 	fd_set rfds;
 	struct timeval tv;
@@ -146,7 +177,7 @@ _("\n"
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDLINUX ($Revision: 1.8 $)";
+const char * const gnupgext_version = "RNDLINUX ($Revision: 1.9.2.1 $)";
 
 static struct {
     int class;
