@@ -1,11 +1,50 @@
 /****************************************************************************
  *									    *
- *   BeOS Randomness-Gathering Code					    *
- *   Copyright Peter Gutmann, Paul Kendall, and Chris Wedgwood 1996-1998    *
- *   Copyright (C) 1998, 1999  Werner Koch
+ *									    *
+ *   Unix Randomness-Gathering Code					    *
+ *									    *
+ *   Copyright Peter Gutmann, Paul Kendall, and Chris Wedgwood 1996-1999.   *
+ *   Heavily modified for GnuPG by Werner Koch				    *
+ *									    *
  *									    *
  ****************************************************************************/
 
+/* This module is part of the cryptlib continuously seeded pseudorandom
+   number generator.  For usage conditions, see lib_rand.c
+
+   [Here is the notice from lib_rand.c:]
+
+   This module and the misc/rnd*.c modules represent the cryptlib
+   continuously seeded pseudorandom number generator (CSPRNG) as described in
+   my 1998 Usenix Security Symposium paper "The generation of random numbers
+   for cryptographic purposes".
+
+   The CSPRNG code is copyright Peter Gutmann (and various others) 1996,
+   1997, 1998, 1999, all rights reserved.  Redistribution of the CSPRNG
+   modules and use in source and binary forms, with or without modification,
+   are permitted provided that the following conditions are met:
+
+   1. Redistributions of source code must retain the above copyright notice
+      and this permission notice in its entirety.
+
+   2. Redistributions in binary form must reproduce the copyright notice in
+      the documentation and/or other materials provided with the distribution.
+
+   3. A copy of any bugfixes or enhancements made must be provided to the
+      author, <pgut001@cs.auckland.ac.nz> to allow them to be added to the
+      baseline version of the code.
+
+  ALTERNATIVELY, the code may be distributed under the terms of the GNU
+  General Public License, version 2 or any later version published by the
+  Free Software Foundation, in which case the provisions of the GNU GPL are
+  required INSTEAD OF the above restrictions.
+
+  Although not required under the terms of the GPL, it would still be nice if
+  you could make any changes available to the author to allow a consistent
+  code base to be maintained */
+
+
+
 /* General includes */
 
 #include <config.h>
@@ -36,9 +75,9 @@
 #ifndef __QNX__
 #include <sys/resource.h>
 #endif				/* __QNX__ */
-#ifdef _AIX
+#if defined( _AIX ) || defined( __QNX__ )
 #include <sys/select.h>
-#endif				/* _AIX */
+#endif				/* _AIX || __QNX__ */
 #ifndef __QNX__
 #include <sys/shm.h>
 #include <sys/signal.h>
@@ -50,6 +89,10 @@
 #endif				/* __hpux 9.x, after that it's in unistd.h */
 #include <sys/wait.h>
 /* #include <kitchensink.h> */
+#ifdef __QNX__
+#include <signal.h>
+#include <process.h>
+#endif		      /* __QNX__ */
 #include <errno.h>
 
 #include "types.h"  /* for byte and u32 typedefs */
@@ -59,7 +102,13 @@
 #include "util.h"
 
 #ifndef EAGAIN
-  #define EAGAIN  EWOULDBLOCK
+#define EAGAIN	EWOULDBLOCK
+#endif
+#ifndef STDIN_FILENO
+#define STDIN_FILENO 0
+#endif
+#ifndef STDOUT_FILENO
+#define STDOUT_FILENO 1
 #endif
 
 #define GATHER_BUFSIZE		49152	/* Usually about 25K are filled */
@@ -265,6 +314,34 @@ typedef struct {
     char data[500];	/* gathered data */
 } GATHER_MSG;
 
+
+#ifndef HAVE_WAITPID
+pid_t
+waitpid(pid_t pid, int *statptr, int options)
+{
+     #ifdef HAVE_WAIT4
+	return wait4(pid, statptr, options, NULL);
+     #else
+	/* If wait4 is also not available, try wait3 for SVR3 variants */
+	/* Less ideal because can't actually request a specific pid */
+	/* For that reason, first check to see if pid is for an */
+	/*   existing process. */
+	int tmp_pid, dummystat;;
+	if (kill(pid, 0) == -1) {
+		errno = ECHILD;
+		return -1;
+	}
+	if (statptr == NULL)
+		statptr = &dummystat;
+	while (((tmp_pid = wait3(statptr, options, 0)) != pid) &&
+		    (tmp_pid != -1) && (tmp_pid != 0) && (pid != -1))
+	    ;
+	return tmp_pid;
+     #endif
+}
+#endif
+
+
 /* Under SunOS popen() doesn't record the pid of the child process.  When
  * pclose() is called, instead of calling waitpid() for the correct child, it
  * calls wait() repeatedly until the right child is reaped.  The problem is
@@ -335,7 +412,9 @@ my_popen(struct RI *entry)
      * close on exec, so new children won't see it */
     close(pipedes[STDOUT_FILENO]);
 
+#ifdef FD_CLOEXEC
     fcntl(pipedes[STDIN_FILENO], F_SETFD, FD_CLOEXEC);
+#endif
 
     stream = fdopen(pipedes[STDIN_FILENO], "r");
 
@@ -428,8 +507,6 @@ slow_poll(FILE *dbgfp, int dbgall, size_t *nbytes )
 		maxFD = dataSources[i].pipeFD;
 	  #ifdef O_NONBLOCK /* Ohhh what a hack (used for Atari) */
 	    fcntl(dataSources[i].pipeFD, F_SETFL, O_NONBLOCK);
-	  #else
-	    #warning O_NONBLOCK is missing
 	  #endif
 	    FD_SET(dataSources[i].pipeFD, &fds);
 	    dataSources[i].length = 0;
@@ -575,6 +652,7 @@ start_gatherer( int pipefd )
     }
     /* close all files but the ones we need */
     {	int nmax, n1, n2, i;
+      #ifdef _SC_OPEN_MAX
 	if( (nmax=sysconf( _SC_OPEN_MAX )) < 0 ) {
 	  #ifdef _POSIX_OPEN_MAX
 	    nmax = _POSIX_OPEN_MAX;
@@ -582,6 +660,9 @@ start_gatherer( int pipefd )
 	    nmax = 20; /* assume a reasonable value */
 	  #endif
 	}
+      #else
+	nmax = 20; /* assume a reasonable value */
+      #endif
 	n1 = fileno( stderr );
 	n2 = dbgfp? fileno( dbgfp ) : -1;
 	for(i=0; i < nmax; i++ ) {
@@ -677,6 +758,10 @@ read_a_msg( int fd, GATHER_MSG *msg )
 }
 
 
+/****************
+ * Using a level of 0 should never block and better add nothing
+ * to the pool.  So this is just a dummy for this gatherer.
+ */
 static int
 gather_random( void (*add)(const void*, size_t, int), int requester,
 	       size_t length, int level )
@@ -686,6 +771,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
     GATHER_MSG msg;
     size_t n;
 
+    if( !level )
+	return 0;
+
     if( !gatherer_pid ) {
 	/* make sure we are not setuid */
 	if( getuid() != geteuid() )
@@ -757,7 +845,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.13 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.14.2.3 $)";
 
 
 static struct {
