@@ -1,5 +1,5 @@
-/* maPPPPinproc.c - handle packets
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+/* mainproc.c - handle packets
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -225,10 +225,14 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 	|| is_RSA(enc->pubkey_algo)  ) {
 	if ( !c->dek && ((!enc->keyid[0] && !enc->keyid[1])
 			  || !seckey_available( enc->keyid )) ) {
-	    c->dek = m_alloc_secure( sizeof *c->dek );
-	    if( (result = get_session_key( enc, c->dek )) ) {
-		/* error: delete the DEK */
-		m_free(c->dek); c->dek = NULL;
+	    if( opt.list_only )
+		result = -1;
+	    else {
+		c->dek = m_alloc_secure( sizeof *c->dek );
+		if( (result = get_session_key( enc, c->dek )) ) {
+		    /* error: delete the DEK */
+		    m_free(c->dek); c->dek = NULL;
+		}
 	    }
 	}
 	else
@@ -268,6 +272,8 @@ print_failed_pkenc( struct kidlist_item *list )
 	PKT_public_key *pk = m_alloc_clear( sizeof *pk );
 	const char *algstr = pubkey_algo_to_string( list->pubkey_algo );
 
+	if( !algstr )
+	    algstr = "[?]";
 	pk->pubkey_algo = list->pubkey_algo;
 	if( !get_pubkey( pk, list->kid ) ) {
 	    size_t n;
@@ -310,9 +316,15 @@ proc_encrypted( CTX c, PACKET *pkt )
 
     print_failed_pkenc( c->failed_pkenc );
 
+    write_status( STATUS_BEGIN_DECRYPTION );
+
     /*log_debug("dat: %sencrypted data\n", c->dek?"":"conventional ");*/
-    if( !c->dek && !c->last_was_session_key ) {
-	/* assume this is old conventional encrypted data */
+    if( opt.list_only )
+	result = -1;
+    else if( !c->dek && !c->last_was_session_key ) {
+	/* assume this is old conventional encrypted data
+	 * Actually we should use IDEA and MD5 in this case, but because
+	 * IDEA is patented we can't do so */
 	c->dek = passphrase_to_dek( NULL, 0,
 		    opt.def_cipher_algo ? opt.def_cipher_algo
 					: DEFAULT_CIPHER_ALGO, NULL, 0 );
@@ -321,6 +333,7 @@ proc_encrypted( CTX c, PACKET *pkt )
 	result = G10ERR_NO_SECKEY;
     if( !result )
 	result = decrypt_data( c, pkt->pkt.encrypted, c->dek );
+
     m_free(c->dek); c->dek = NULL;
     if( result == -1 )
 	;
@@ -338,11 +351,12 @@ proc_encrypted( CTX c, PACKET *pkt )
     else {
 	write_status( STATUS_DECRYPTION_FAILED );
 	log_error(_("decryption failed: %s\n"), g10_errstr(result));
-	/* Hmmm: does this work when we have encrypted using a multiple
+	/* Hmmm: does this work when we have encrypted using multiple
 	 * ways to specify the session key (symmmetric and PK)*/
     }
     free_packet(pkt);
     c->last_was_session_key = 0;
+    write_status( STATUS_END_DECRYPTION );
 }
 
 
@@ -395,12 +409,14 @@ proc_plaintext( CTX c, PACKET *pkt )
 		clearsig = 1;
 	}
     }
-    if( !any ) { /* no onepass sig packet: enable all standard algos */
+
+    if( !any && !opt.skip_verify ) {
+	/* no onepass sig packet: enable all standard algos */
 	md_enable( c->mfx.md, DIGEST_ALGO_RMD160 );
 	md_enable( c->mfx.md, DIGEST_ALGO_SHA1 );
 	md_enable( c->mfx.md, DIGEST_ALGO_MD5 );
     }
-    if( only_md5 ) {
+    if( opt.pgp2_workarounds && only_md5 && !opt.skip_verify ) {
 	/* This is a kludge to work around a bug in pgp2.  It does only
 	 * catch those mails which are armored.  To catch the non-armored
 	 * pgp mails we could see whether there is the signature packet
@@ -408,10 +424,11 @@ proc_plaintext( CTX c, PACKET *pkt )
 	 */
 	c->mfx.md2 = md_open( DIGEST_ALGO_MD5, 0);
     }
-  #if 0
-    #warning md_start_debug is enabled
-    md_start_debug( c->mfx.md, "verify" );
-  #endif
+    if ( DBG_HASHING ) {
+	md_start_debug( c->mfx.md, "verify" );
+	if ( c->mfx.md2  )
+	    md_start_debug( c->mfx.md2, "verify2" );
+    }
     rc = handle_plaintext( pt, &c->mfx, c->sigs_only, clearsig );
     if( rc == G10ERR_CREATE_FILE && !c->sigs_only) {
 	/* can't write output but we hash it anyway to
@@ -505,6 +522,11 @@ do_check_sig( CTX c, KBNODE node, int *is_selfsig )
 	    || c->list->pkt->pkttype == PKT_PUBLIC_SUBKEY ) {
 	    return check_key_signature( c->list, node, is_selfsig );
 	}
+	else if( sig->sig_class == 0x20 ) {
+	    log_info(_("standalone revocation - "
+		       "use \"gpg --import\" to apply\n"));
+	    return G10ERR_NOT_PROCESSED;
+	}
 	else {
 	    log_error("invalid root packet for sigclass %02x\n",
 							sig->sig_class);
@@ -532,8 +554,12 @@ print_userid( PACKET *pkt )
 	printf("ERROR: unexpected packet type %d", pkt->pkttype );
 	return;
     }
-    print_string( stdout,  pkt->pkt.user_id->name, pkt->pkt.user_id->len,
-							opt.with_colons );
+    if( opt.with_colons )
+	print_string( stdout,  pkt->pkt.user_id->name,
+				pkt->pkt.user_id->len, ':');
+    else
+	print_utf8_string( stdout,  pkt->pkt.user_id->name,
+				     pkt->pkt.user_id->len );
 }
 
 
@@ -636,11 +662,13 @@ list_node( CTX c, KBNODE node )
 	    keyid_from_pk( pk, keyid );
 	    if( mainkey ) {
 		c->local_id = pk->local_id;
-		c->trustletter = query_trust_info( pk, NULL );
+		c->trustletter = opt.fast_list_mode?
+					   0 : query_trust_info( pk, NULL );
 	    }
-	    printf("%s:%c:%u:%d:%08lX%08lX:%s:%s:",
-		    mainkey? "pub":"sub",
-		    c->trustletter,
+	    printf("%s:", mainkey? "pub":"sub" );
+	    if( c->trustletter )
+		putchar( c->trustletter );
+	    printf(":%u:%d:%08lX%08lX:%s:%s:",
 		    nbits_from_pk( pk ),
 		    pk->pubkey_algo,
 		    (ulong)keyid[0],(ulong)keyid[1],
@@ -649,7 +677,7 @@ list_node( CTX c, KBNODE node )
 	    if( c->local_id )
 		printf("%lu", c->local_id );
 	    putchar(':');
-	    if( c->local_id )
+	    if( c->local_id && !opt.fast_list_mode )
 		putchar( get_ownertrust_info( c->local_id ) );
 	    putchar(':');
 	    if( node->next && node->next->pkt->pkttype == PKT_RING_TRUST) {
@@ -667,6 +695,7 @@ list_node( CTX c, KBNODE node )
 				      pubkey_letter( pk->pubkey_algo ),
 				      (ulong)keyid_from_pk( pk, NULL ),
 				      datestr_from_pk( pk )	);
+
 	if( mainkey ) {
 	    /* and now list all userids with their signatures */
 	    for( node = node->next; node; node = node->next ) {
@@ -709,6 +738,10 @@ list_node( CTX c, KBNODE node )
 		}
 	    }
 	}
+	else if( pk->expiredate ) { /* of subkey */
+	    printf(_(" [expires: %s]"), expirestr_from_pk( pk ) );
+	}
+
 	if( !any )
 	    putchar('\n');
 	if( !mainkey && opt.fingerprint > 1 )
@@ -821,7 +854,8 @@ list_node( CTX c, KBNODE node )
 	    putchar(':');
 	    if( sigrc != ' ' )
 		putchar(sigrc);
-	    printf(":::%08lX%08lX:%s::::", (ulong)sig->keyid[0],
+	    printf("::%d:%08lX%08lX:%s::::", sig->pubkey_algo,
+					     (ulong)sig->keyid[0],
 		       (ulong)sig->keyid[1], datestr_from_sig(sig));
 	}
 	else
@@ -838,7 +872,7 @@ list_node( CTX c, KBNODE node )
 	    if( opt.with_colons )
 		putchar(':');
 	}
-	else {
+	else if( !opt.fast_list_mode ) {
 	    p = get_user_id( sig->keyid, &n );
 	    print_string( stdout, p, n, opt.with_colons );
 	    m_free(p);
@@ -852,6 +886,7 @@ list_node( CTX c, KBNODE node )
 }
 
 
+
 int
 proc_packets( void *anchor, IOBUF a )
 {
@@ -864,6 +899,8 @@ proc_packets( void *anchor, IOBUF a )
     return rc;
 }
 
+
+
 int
 proc_signature_packets( void *anchor, IOBUF a,
 			STRLIST signedfiles, const char *sigfilename )
@@ -1032,7 +1069,7 @@ check_sig_and_print( CTX c, KBNODE node )
 	    (int)strlen(tstr), tstr, astr? astr: "?", (ulong)sig->keyid[1] );
 
     rc = do_check_sig(c, node, NULL );
-    if( rc == G10ERR_NO_PUBKEY && opt.keyserver_name ) {
+    if( rc == G10ERR_NO_PUBKEY && opt.keyserver_name && opt.auto_key_retrieve) {
 	if( !hkp_ask_import( sig->keyid ) )
 	    rc = do_check_sig(c, node, NULL );
     }
@@ -1112,7 +1149,8 @@ check_sig_and_print( CTX c, KBNODE node )
 	    buf[16] = 0;
 	    write_status_text( STATUS_NO_PUBKEY, buf );
 	}
-	log_error(_("Can't check signature: %s\n"), g10_errstr(rc) );
+	if( rc != G10ERR_NOT_PROCESSED )
+	    log_error(_("Can't check signature: %s\n"), g10_errstr(rc) );
     }
     return rc;
 }
@@ -1127,7 +1165,7 @@ proc_tree( CTX c, KBNODE node )
     KBNODE n1;
     int rc;
 
-    if( opt.list_packets )
+    if( opt.list_packets || opt.list_only )
 	return;
 
     c->local_id = 0;
@@ -1153,13 +1191,16 @@ proc_tree( CTX c, KBNODE node )
 		md_enable( c->mfx.md, n1->pkt->pkt.signature->digest_algo);
 	    }
 	    /* ask for file and hash it */
-	    if( c->sigs_only )
+	    if( c->sigs_only ) {
 		rc = hash_datafiles( c->mfx.md, NULL,
 				     c->signed_data, c->sigfilename,
 			n1? (n1->pkt->pkt.onepass_sig->sig_class == 0x01):0 );
-	    else
+	    }
+	    else {
 		rc = ask_for_detached_datafile( c->mfx.md, c->mfx.md2,
-					    iobuf_get_fname(c->iobuf), 0 );
+						iobuf_get_fname(c->iobuf),
+			n1? (n1->pkt->pkt.onepass_sig->sig_class == 0x01):0 );
+	    }
 	    if( rc ) {
 		log_error("can't hash datafile: %s\n", g10_errstr(rc));
 		return;
@@ -1172,12 +1213,17 @@ proc_tree( CTX c, KBNODE node )
     else if( node->pkt->pkttype == PKT_SIGNATURE ) {
 	PKT_signature *sig = node->pkt->pkt.signature;
 
-	if( !c->have_data ) {
+	if( sig->sig_class != 0x00 && sig->sig_class != 0x01 )
+	    log_info(_("standalone signature of class 0x%02x\n"),
+						    sig->sig_class);
+	else if( !c->have_data ) {
 	    /* detached signature */
 	    free_md_filter_context( &c->mfx );
 	    c->mfx.md = md_open(sig->digest_algo, 0);
-	    if( sig->digest_algo == DIGEST_ALGO_MD5
-		&& is_RSA( sig->pubkey_algo ) ) {
+	    if( !opt.pgp2_workarounds )
+		;
+	    else if( sig->digest_algo == DIGEST_ALGO_MD5
+		     && is_RSA( sig->pubkey_algo ) ) {
 		/* enable a workaround for a pgp2 bug */
 		c->mfx.md2 = md_open( DIGEST_ALGO_MD5, 0 );
 	    }
@@ -1188,6 +1234,7 @@ proc_tree( CTX c, KBNODE node )
 		 * signature has been created in textmode */
 		c->mfx.md2 = md_open( sig->digest_algo, 0 );
 	    }
+	  #if 0 /* workaround disabled */
 	    /* Here we have another hack to work around a pgp 2 bug
 	     * It works by not using the textmode for detached signatures;
 	     * this will let the first signature check (on md) fail
@@ -1195,14 +1242,18 @@ proc_tree( CTX c, KBNODE node )
 	     * then produce the "correct" hash.  This is very, very ugly
 	     * hack but it may help in some cases (and break others)
 	     */
-	    if( c->sigs_only )
+		    /*	c->mfx.md2? 0 :(sig->sig_class == 0x01) */
+	  #endif
+	    if( c->sigs_only ) {
 		rc = hash_datafiles( c->mfx.md, c->mfx.md2,
 				     c->signed_data, c->sigfilename,
-				     c->mfx.md2? 0 :(sig->sig_class == 0x01) );
-	    else
+				     (sig->sig_class == 0x01) );
+	    }
+	    else {
 		rc = ask_for_detached_datafile( c->mfx.md, c->mfx.md2,
-					 iobuf_get_fname(c->iobuf),
-			       c->mfx.md2? 0 :(sig->sig_class == 0x01) );
+						iobuf_get_fname(c->iobuf),
+						(sig->sig_class == 0x01) );
+	    }
 	    if( rc ) {
 		log_error("can't hash datafile: %s\n", g10_errstr(rc));
 		return;
