@@ -1,5 +1,5 @@
 /* misc.c -  miscellaneous functions
- *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -41,7 +41,10 @@ const char *g10m_revision_string(int);
 const char *g10c_revision_string(int);
 const char *g10u_revision_string(int);
 
-volatile void
+#ifdef __GNUC__
+volatile
+#endif
+	 void
 pull_in_libs(void)
 {
     g10m_revision_string(0);
@@ -79,6 +82,7 @@ trap_unaligned(void)
 void
 disable_core_dumps()
 {
+ #ifndef HAVE_DOSISH_SYSTEM
   #ifdef HAVE_SETRLIMIT
     struct rlimit limit;
 
@@ -91,6 +95,7 @@ disable_core_dumps()
   #endif
     if( !opt.quiet )
 	log_info(_("WARNING: program may create a core file!\n"));
+ #endif
 }
 
 
