@@ -1,5 +1,5 @@
 /* passphrase.c -  Get a passphrase
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -114,9 +114,11 @@ read_passphrase_from_fd( int fd )
  *	    (only for mode 2)
  *	    a dek->keylen of 0 means: no passphrase entered.
  *	    (only for mode 2)
+ * pubkey_algo is only informational.
  */
 DEK *
-passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
+passphrase_to_dek( u32 *keyid, int pubkey_algo,
+		   int cipher_algo, STRING2KEY *s2k, int mode )
 {
     char *pw = NULL;
     DEK *dek;
@@ -139,8 +141,8 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 	    sprintf( buf, "%08lX%08lX", (ulong)keyid[0], (ulong)keyid[1] );
 	    if( keyid[2] && keyid[3] && keyid[0] != keyid[2]
 				     && keyid[1] != keyid[3] )
-		sprintf( buf+strlen(buf), " %08lX%08lX",
-					  (ulong)keyid[2], (ulong)keyid[3] );
+		sprintf( buf+strlen(buf), " %08lX%08lX %d 0",
+			   (ulong)keyid[2], (ulong)keyid[3], pubkey_algo );
 	    write_status_text( STATUS_NEED_PASSPHRASE, buf );
 	}
 	else {
@@ -157,7 +159,7 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 	tty_printf(_("\nYou need a passphrase to unlock the secret key for\n"
 		     "user: \"") );
 	p = get_user_id( keyid, &n );
-	tty_print_string( p, n );
+	tty_print_utf8_string( p, n );
 	m_free(p);
 	tty_printf("\"\n");
 
@@ -185,8 +187,8 @@ passphrase_to_dek( u32 *keyid, int cipher_algo, STRING2KEY *s2k, int mode )
 	strcpy( pw, fd_passwd );
     }
     else if( opt.batch ) {
-	write_status( STATUS_MISSING_PASSPHRASE );
-	log_fatal("Can't query password in batchmode\n");
+	log_error(_("can't query password in batchmode\n"));
+	pw = m_strdup( "" ); /* return an empty passphrase */
     }
     else {
 	pw = cpr_get_hidden("passphrase.enter", _("Enter passphrase: ") );
@@ -240,7 +242,13 @@ hash_passphrase( DEK *dek, char *pw, STRING2KEY *s2k, int create )
     md = md_open( s2k->hash_algo, 1);
     for(pass=0; used < dek->keylen ; pass++ ) {
 	if( pass ) {
-	    md_reset(md);
+	    if( (opt.emulate_bugs & EMUBUG_3DESS2K)) {
+		int tmp = md->finalized;
+		md_reset( md );
+		md->finalized = tmp;
+	    }
+	    else
+		md_reset(md);
 	    for(i=0; i < pass; i++ ) /* preset the hash context */
 		md_putc(md, 0 );
 	}
