@@ -1,5 +1,5 @@
 /* textfilter.c
- *	Copyright (C) 1998,1999 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999, 2000 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -31,6 +31,7 @@
 #include "util.h"
 #include "filter.h"
 #include "i18n.h"
+#include "options.h"
 
 #ifdef HAVE_DOSISH_SYSTEM
   #define LF "\r\n"
@@ -156,6 +157,9 @@ copy_clearsig_text( IOBUF out, IOBUF inp, MD_HANDLE md,
     int truncated = 0;
     int pending_lf = 0;
 
+    if( !opt.pgp2_workarounds )
+	pgp2mode = 0;
+
     if( !escape_dash )
 	escape_from = 0;
 
