@@ -80,6 +80,8 @@ static int underflow(IOBUF a);
  * IOBUFCTRL_FLUSH: called by iobuf_flush() to write out the collected stuff.
  *		    *RET_LAN is the number of bytes in BUF.
  *
+ * IOBUFCTRL_CANCEL: send to all filters on behalf of iobuf_cancel.  The
+ *		    filter may take appropriate action on this message.
  */
 static int
 file_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
@@ -168,7 +170,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    break;
 		}
 		else if( a->partial ) {
-		    /* These OpenPGP introduced huffman encoded length
+		    /* These OpenPGP introduced huffman like encoded length
 		     * bytes are really a mess :-( */
 		    if( a->first_c ) {
 			c = a->first_c;
@@ -220,6 +222,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		    else { /* next partial body length */
 			a->size = 1 << (c & 0x1f);
 		    }
+	    /*	log_debug("partial: ctx=%p c=%02x size=%u\n", a, c, a->size);*/
 		}
 		else { /* the gnupg partial length scheme - much better :-) */
 		    c = iobuf_get(chain);
@@ -372,6 +375,7 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 		 */
 		/* construct header */
 		len = a->buflen;
+		/*log_debug("partial: remaining length=%u\n", len );*/
 		if( len < 192 )
 		    rc = iobuf_put(chain, len );
 		else if( len < 8384 ) {
@@ -495,13 +499,41 @@ int
 iobuf_cancel( IOBUF a )
 {
     const char *s;
+    IOBUF a2;
+    int rc;
+  #ifdef HAVE_DOSISH_SYSTEM
+    char *remove_name = NULL;
+  #endif
 
     if( a && a->use == 2 ) {
 	s = iobuf_get_real_fname(a);
-	if( s && *s )
-	    remove(s);	/* remove the file. Fixme: this will fail for MSDOZE*/
-    }			/* because the file is still open */
-    return iobuf_close(a);
+	if( s && *s ) {
+	  #ifdef HAVE_DOSISH_SYSTEM
+	    remove_name = m_strdup ( s );
+	  #else
+	    remove(s);
+	  #endif
+	}
+    }
+
+    /* send a cancel message to all filters */
+    for( a2 = a; a2 ; a2 = a2->chain ) {
+	size_t dummy;
+	if( a2->filter )
+	    a2->filter( a2->filter_ov, IOBUFCTRL_CANCEL, a2->chain,
+							 NULL, &dummy );
+    }
+
+    rc = iobuf_close(a);
+  #ifdef HAVE_DOSISH_SYSTEM
+    if ( remove_name ) {
+	/* Argg, MSDOS does not allow to remove open files.  So
+	 * we have to do it here */
+	remove ( remove_name );
+	m_free ( remove_name );
+    }
+  #endif
+    return rc;
 }
 
 
