@@ -57,8 +57,8 @@ struct memblock_struct {
 
 
 static void  *pool;
-static int   pool_okay;
-static int   pool_is_mmapped;
+static volatile int pool_okay; /* may be checked in an atexit function */
+static volatile int pool_is_mmapped;
 static size_t poolsize; /* allocated length */
 static size_t poollen;	/* used length */
 static MEMBLOCK *unused_blocks;
@@ -97,8 +97,11 @@ lock_pool( void *p, size_t n )
 	  #ifdef EAGAIN  /* OpenBSD returns this */
 	    && errno != EAGAIN
 	  #endif
+	  #ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
+	    && errno != ENOSYS
+	  #endif
 	  )
-	    log_error("cant lock memory: %s\n", strerror(err));
+	    log_error("can't lock memory: %s\n", strerror(err));
 	show_warning = 1;
     }
 
@@ -125,7 +128,9 @@ lock_pool( void *p, size_t n )
   #endif
 
     if( uid && !geteuid() ) {
-	if( setuid( uid ) || getuid() != geteuid()  )
+	/* check that we really dropped the privs.
+	 * Note: setuid(0) should always fail */
+	if( setuid( uid ) || getuid() != geteuid() || !setuid(0) )
 	    log_fatal("failed to reset uid: %s\n", strerror(errno));
     }
 
@@ -134,11 +139,20 @@ lock_pool( void *p, size_t n )
 	  #ifdef EAGAIN  /* OpenBSD returns this */
 	    && errno != EAGAIN
 	  #endif
+	  #ifdef ENOSYS  /* Some SCOs return this (function not implemented) */
+	    && errno != ENOSYS
+	  #endif
 	  )
-	    log_error("cant lock memory: %s\n", strerror(err));
+	    log_error("can't lock memory: %s\n", strerror(err));
 	show_warning = 1;
     }
 
+  #elif defined ( __QNX__ )
+    /* QNX does not page at all, so the whole secure memory stuff does
+     * not make much sense.  However it is still of use because it
+     * wipes out the memory on a free().
+     * Therefore it is sufficient to suppress the warning
+     */
   #else
     log_info("Please note that you don't have secure memory on this system\n");
   #endif
@@ -248,7 +262,7 @@ secmem_init( size_t n )
 	disable_secmem=1;
 	uid = getuid();
 	if( uid != geteuid() ) {
-	    if( setuid( uid ) || getuid() != geteuid() )
+	    if( setuid( uid ) || getuid() != geteuid() || !setuid(0) )
 		log_fatal("failed to drop setuid\n" );
 	}
       #endif
@@ -370,6 +384,16 @@ m_is_secure( const void *p )
     return p >= pool && p < (void*)((char*)pool+poolsize);
 }
 
+
+
+/****************
+ * Warning:  This code might be called by an interrupt handler
+ *	     and frankly, there should really be such a handler,
+ *	     to make sure that the memory is wiped out.
+ *	     We hope that the OS wipes out mlocked memory after
+ *	     receiving a SIGKILL - it really should do so, otherwise
+ *	     there is no chance to get the secure memory cleaned.
+ */
 void
 secmem_term()
 {
